import java.io.BufferedReader;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

public class ConfigReader {

	/**
	 * Carica la lista di host dal file specificato.
	 * Ogni linea del file contine i dati di un host.
	 * Il numero di porta se omesso si assume uguale a quello di default di ssh (il 22).
	 * La password puo essere omessa ove non necessaria.
	 * 
	 * Es.
	 * 
	 * username@hostname:port password
	 * username2@hostname2 password2
	 * username3@hostname3:port3
	 * 
	 * @return Una lista (possibilmente vuota) contenente gli Host specificati nel file
	 * */
	
	private static String COMMENT_SEPARATOR = "#";
	private static String PASSWORD_SEPARATOR = " ";
	private static String PORT_SEPARATOR = ":";
	private static String HOST_SEPARATOR = "@";
	
	public static List<Host> loadConfig(String filename) throws FileNotFoundException
	{
		if(filename == null)
			throw new IllegalArgumentException("File Name is null");		

		FileInputStream fis = new FileInputStream(filename);			

		BufferedReader br = new BufferedReader(new InputStreamReader(fis));

		List<Host> toReturn = new ArrayList<Host>();

		try {
			String line;
			Host host;
			/*formato linea: "username@hostname:port password"*/

			while ((line = br.readLine()) != null)	{

				line = line.trim();//elimino gli spazi all'inizio e alla fine

				if(line.isEmpty() || line.startsWith(COMMENT_SEPARATOR))//linea vuota o commenti
					continue;

				try{
					host = string2Host(line.trim());
				}
				catch (IllegalArgumentException e) {//string null o nel formato scorretto
					continue;				
				}

				//tutto ok aggiungo alla lista
				toReturn.add(host);							
			}
		} catch (IOException e) {
			e.printStackTrace();
		}

		return toReturn;

	}
	
	/**
	 * Converte la rappresentazione in stringa di un host in un oggetto Host.
	 * il formato della di hostString e':
	 * 
	 * username@hostname:port password
	 * 
	 * Password puo' essere omesso.
	 *  
	 * @throws IllegalArgumentException Se hostString e' null o non e' nel formato previsto
	 * */
	private static Host string2Host(String hostString)
	{
		/*formato hostString: "username@hostname:port password"*/
		
		if(hostString == null)
			throw new IllegalArgumentException("HostString is null");	
		
		String hostname = "localhost";
		String username = "user";
		String password = "";
		
		int port = -1;
		
		String[] firstSplit = hostString.split(PASSWORD_SEPARATOR); /* {username@hostname:port, password}*/
				
		try{
			password = firstSplit[1];
		}
		catch (ArrayIndexOutOfBoundsException e) {
		}//password non indicata
							
		String[] secondSplit = firstSplit[0].split(PORT_SEPARATOR); // {username@hostname, port}
			
		try{
			port = Integer.parseInt(secondSplit[1]);
		}
		catch (ArrayIndexOutOfBoundsException e) {}//porta non indicata

		String[] thirdSplit = secondSplit[0].split(HOST_SEPARATOR); // {username , hostname}

		try{
			username = thirdSplit[0];
	
			hostname = thirdSplit[1];
		}
		catch (ArrayIndexOutOfBoundsException e) {
			throw new IllegalArgumentException("Formato hostString non valido"); 
		}//formato non valido
		
		if(port == -1)//porta non indicata uso quella di default (22)
			return new Host(username, hostname, password);
		else return new Host(username, hostname, password, port);
		
	}

	public static void main(String[] args) {
		
		List<Host> list;
		
		try {
			list = loadConfig("hosts.conf");
			
			for (Host host : list)
			
				System.out.println(host);
	
		} catch (FileNotFoundException e) {
			System.out.println("File di config non trovato");
		}
	}
}
