package examples;
/* This is the muskel: a skeleton/RMI based, parallel programming library
Copyright (C) 2006  Marco Danelutto, Dept. Computer Science, Univ. Pisa, Italy

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */

import skeleton.Compute;

import muskel.*;
import muskel.stream.ConsoleOutputManager;
import muskel.stream.InputManager;
import muskel.stream.IntegerVectorStreamInputManager;
import muskel.stream.OutputManager;

/**
 * Codice di esempio per creare una app applicazione parallela
 * usando muskel.
 *	Lo skeleton Map8 utilizzato è un Map a "grado fisso di parallelismo"
 *	con 8 Square come workers.
 *	Map8 è un ParCompute (cioe' uno skeleton definito dall'utente).
 * @author albanese
 */
public class Map8Example {

	public static void main(String[] args) {
		int parDegree;
		try { parDegree = Integer.parseInt(args[0]);
 		} catch(ArrayIndexOutOfBoundsException e) {
 			//il grado di parallellismo richiesto a muskel
 			parDegree = 1;
 		}
				
 		Manager manager = new Manager(); 

 		Compute map8 = new Map8(manager);
 		
 		//quanti task calcolare
 		int nTasks = 5;
 		//il grado di parallelismo della map
 		int mapdegree = 8;
 		
		InputManager sampleISM = 
			(InputManager) new IntegerVectorStreamInputManager(mapdegree, nTasks);
		OutputManager sampleOSM = 
			(OutputManager) new ConsoleOutputManager();
					
		manager.setInputManager(sampleISM);
		manager.setOutputManager(sampleOSM);
		manager.setContract(new ParDegree(parDegree));
		manager.setProgram(map8);
		
		manager.compute();
	}

}
