package examples;
/* This is the muskel: a skeleton/RMI based, parallel programming library
Copyright (C) 2006  Marco Danelutto, Dept. Computer Science, Univ. Pisa, Italy

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */

import skeleton.*;

import map.MapFactory;
import muskel.*;
import muskel.stream.ConsoleOutputManager;
import muskel.stream.InputManager;
import muskel.stream.IntegerVectorStreamInputManager;
import muskel.stream.OutputManager;

/**
 * Esempio di utilizzo di MapFactory per la creazione skeleton map
 * con grado di parallelismo e tipo di worker specificati
 * 
 * @author albanese
 */

public class MapExample {

	public static void main(String[] args) {
		
		if(args.length != 2) {
			printUsage();
			return;
		}
			
		//il grado di parallellismo richiesto a muskel
		int parDegree = 1;
		
 		//il grado di parallelismo della map
 		int mapDegree = 5;
		
		try {
			mapDegree = Integer.parseInt(args[0]);
			parDegree = Integer.parseInt(args[1]);
			
			if(mapDegree < 1 || parDegree < 1) {
				System.out.println("mapDegree and parDegree must be >= 1");
				return;
			}
				
 		} catch(NumberFormatException e) {
 			printUsage();		
 		}
 		
 		System.out.println("Requested a "+mapDegree+"-degree Map on "+parDegree+" Remote Interpreters.");
 		
 		//il class del worker
 		Class<Square> workerClass = Square.class;
 		 		
 		//quanti task calcolare
 		int nTasks = 1;
 		
 		Manager manager = new Manager(); 

 		Compute map = MapFactory.newMap(manager, workerClass, mapDegree);
 		
 		if(map == null) {
 			System.out.println("Error occured during map creation, sorry.");
 			return;
 		}

		InputManager sampleISM = 
			(InputManager) new IntegerVectorStreamInputManager(mapDegree, nTasks);
		OutputManager sampleOSM = 
			(OutputManager) new ConsoleOutputManager();
					
		manager.setInputManager(sampleISM);
		manager.setOutputManager(sampleOSM);
		manager.setContract(new ParDegree(parDegree));
		manager.setProgram(map);
		
		manager.compute();
	}

	private static void printUsage() {
		
		System.out.println("usage: java examples.MapExample mapDegree parDegree");	
	}
	
}