package examples;
/* This is the muskel: a skeleton/RMI based, parallel programming library
Copyright (C) 2006  Marco Danelutto, Dept. Computer Science, Univ. Pisa, Italy

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */

import skeleton.Compute;
import skeleton.Pipeline;
import muskel.*;
import muskel.stream.ConsoleOutputManager;
import muskel.stream.InputManager;
import muskel.stream.IntegerStreamInputManager;
import muskel.stream.NonBlockingInputManager;
import muskel.stream.OutputManager;
import muskel.stream.DynamicInputManager;

/**
 * Utilizzo tipo di SECompute in congiunzione con un DynamicInputManager che
 * supporta aggiunta di Task a runtime.
 * 
 * @author albanese
 *
 */
public class SESampleCode {

	public static void main(String[] args) {
		int parDegree;
		try { parDegree = Integer.parseInt(args[0]);
 		} catch(ArrayIndexOutOfBoundsException e) { parDegree = 1; }

		Compute sq1 = new SESquare();
		Compute sq2 = new SESquare();

		Compute main = new Pipeline(sq1,sq2);

		int numTask = 10;
		
		NonBlockingInputManager sampleISM = new IntegerStreamInputManager(numTask);
		
		InputManager sampleSeISM = new DynamicInputManager(sampleISM);		
		
		OutputManager sampleOSM = 
			(OutputManager) new ConsoleOutputManager();
		
		Manager manager = new Manager(main, sampleSeISM, sampleOSM);
		manager.setContract(new ParDegree(parDegree));
		
		manager.compute();
	}

}
