package examples;
/* This is the muskel: a skeleton/RMI based, parallel programming library
Copyright (C) 2006  Marco Danelutto, Dept. Computer Science, Univ. Pisa, Italy

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */

import skeleton.Compute;
import skeleton.ParCompute;
import muskel.*;
import muskel.mdf.*;
import muskel.stream.ConsoleOutputManager;
import muskel.stream.InputManager;
import muskel.stream.IntegerStreamInputManager;
import muskel.stream.OutputManager;

/**
 * Codice di esempio per la costruzione di una applicazione usando muskel.
 * 
 * L'esempio crea un grafo MDF che non può essere il risultato
 * di annidamenti arbitrari di pipe e farm.
 * 
 * @author marcod
 *
 */
public class SampleMdf2 {

	public static void main(String[] args) {
		
		int parDegree;
		try { parDegree = Integer.parseInt(args[0]);
 		} catch(ArrayIndexOutOfBoundsException e) { parDegree = 1; }

		Manager manager = new Manager(); 

		Compute inc1 = new Splitter();
		
		IntegerIdGenerator intIdGen = new IntegerIdGenerator();
		
		InstructionId i1_Id = new InstructionId(intIdGen.generateId());
		InstructionId i2_Id = new InstructionId(intIdGen.generateId());
		InstructionId i3_Id = new InstructionId(intIdGen.generateId());
		InstructionId i4_Id = new InstructionId(intIdGen.generateId());
		
		Destination d1 = new Destination(
				new InputPositionId(new IntegerIdentifier(0)), i2_Id, Mdfi.NoGraphId);

		Destination d2 = new Destination(
				new InputPositionId(new IntegerIdentifier(0)), i3_Id, Mdfi.NoGraphId);
				
		Destination[] dests = {d1, d2};
		Mdfi i1 = new Mdfi(manager,i1_Id,inc1,1,2,dests);
		
		Compute sq1 = new Square();
		
		Destination d3 = new Destination(
				new InputPositionId(new IntegerIdentifier(0)), i4_Id, Mdfi.NoGraphId);
		
		Destination[] dests1 = {d3};
		
		Mdfi i2 = new Mdfi(manager, i2_Id, sq1, 1, 1, dests1);
		
		Compute sq2 = new Square();
		
		Destination d4 = new Destination(
				new InputPositionId(new IntegerIdentifier(1)), i4_Id, Mdfi.NoGraphId);
		
		Destination[] dests2 = {d4};
		Mdfi i3 = new Mdfi(manager, i3_Id, sq2, 1, 1, dests2);
		
		Compute add = new Comp();
		
		Destination d5 = new Destination(
				new InputPositionId(new IntegerIdentifier(0)), Mdfi.NoInstrId, Mdfi.NoGraphId);
		
		Destination[] dests3 = {d5};
		
		Mdfi i4 = new Mdfi(manager, i4_Id, add, 2, 1, dests3);		
				
		MdfGraph graph = new MdfGraph();
		
		graph.setInputInstruction(i1);
		graph.addInstruction(i1);
		graph.addInstruction(i2);
		graph.addInstruction(i3);
		graph.addInstruction(i4);
		graph.setOutputInstruction(i4);
		
		ParCompute main = new ParCompute(graph);
		
		InputManager sampleISM = 
			(InputManager) new IntegerStreamInputManager(10);
		OutputManager sampleOSM = 
			(OutputManager) new ConsoleOutputManager();
		
		manager.setInputManager(sampleISM);
		manager.setOutputManager(sampleOSM);
		manager.setContract(new ParDegree(parDegree));
		manager.setProgram(main);
		
		manager.compute();
	}

}
