package examples;
/* This is the muskel: a skeleton/RMI based, parallel programming library
Copyright (C) 2006  Marco Danelutto, Dept. Computer Science, Univ. Pisa, Italy

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */

import skeleton.Compute;
import skeleton.ParCompute;
import muskel.*;
import muskel.mdf.Destination;
import muskel.mdf.InputPositionId;
import muskel.mdf.InstructionId;
import muskel.mdf.IntegerIdGenerator;
import muskel.mdf.IntegerIdentifier;
import muskel.mdf.MdfGraph;
import muskel.mdf.Mdfi;
import muskel.stream.ConsoleOutputManager;
import muskel.stream.InputManager;
import muskel.stream.IntegerStreamInputManager;
import muskel.stream.OutputManager;

/**
 * This shows how to build a parallel application using muskel.
 * This example runs with version May06 of muskel (or later);
 * 
 * @author marcod
 *
 */
public class SampleMdfg {

	public static void main(String[] args) {

		
		int parDegree;
		try { parDegree = Integer.parseInt(args[0]);
 		} catch(ArrayIndexOutOfBoundsException e) { parDegree = 1; }

		Manager manager = new Manager(); 

		IntegerIdGenerator intIdGen = new IntegerIdGenerator();
		
		InstructionId i1_Id = new InstructionId(intIdGen.generateId());
		InstructionId i2_Id = new InstructionId(intIdGen.generateId());
	
		Compute inc1 = new Inc();
		
		Destination d = new Destination(
				new InputPositionId(new IntegerIdentifier(0)), i2_Id, Mdfi.NoGraphId);
		
		Destination[] dests = {d};
		
		Mdfi i1 = new Mdfi(manager, i1_Id, inc1, 1, 1, dests);
		
		Compute sq1 = new Square();
		
		Destination d1 = new Destination(
				new InputPositionId(new IntegerIdentifier(0)), Mdfi.NoInstrId, Mdfi.NoGraphId);
		
		Destination[] dests1 = {d1};
		
		Mdfi i2 = new Mdfi(manager, i2_Id, sq1, 1, 1, dests1);
		
		MdfGraph graph = new MdfGraph();
		
		graph.setInputInstruction(i1);
		graph.addInstruction(i1);
		graph.addInstruction(i2);
		graph.setOutputInstruction(i2);
		
		ParCompute main = new ParCompute(graph);

		InputManager sampleISM = 
			(InputManager) new IntegerStreamInputManager(10);
		OutputManager sampleOSM = 
			(OutputManager) new ConsoleOutputManager();
		
		manager.setInputManager(sampleISM);
		manager.setOutputManager(sampleOSM);
		manager.setContract(new ParDegree(parDegree));
		manager.setProgram(main);
		
		manager.compute();
	}

}
