/* This is the muskel: a skeleton/RMI based, parallel programming library
Copyright (C) 2006  Marco Danelutto, Dept. Computer Science, Univ. Pisa, Italy

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */

/*
 * Created on Mar 26, 2004
 *
 * To change the template for this generated file go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
package muskel;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.InetAddress;
import java.net.MulticastSocket;
import java.net.SocketTimeoutException;
import org.apache.log4j.Logger;

/**
 * The discovery Thread always looks for new workers.
 * Every time a worker is discovered, its name is issued
 * to the Discovery Service that provides storing
 * it properly in the data structure holding all the 
 * available processing resources. 
 * <br>
 * <p>
 * 
 * @author Danelutto Marco
 * 
 */
public class DiscoveryThread extends Thread {
	
	/** the service used to keep track of the workers found */
	DiscoveryService service = null;
	
	/** 
	 * constructor
	 * @param s the discovery service to be used to
	 * 			store the (possibly new) workers found
	 */
	public DiscoveryThread(DiscoveryService s) {
		service = s;
		return;
	}
	
	/**
	 * this is to find workers with the discovery mechanism instead.
	 * Assumes that the fastest
	 * responders will be the best ones and therefore put them in front of the
	 * list.
	 * <br>
	 * The TTL of the multicast socket used to send
	 * discovery messages is currently set to a very large value,
	 * in order to be able to reach the larger number of nodes possible. <br>
	 * Take into account that usually firewalls of autonomous systems
	 * will filter out the discovery messages, being directed
	 * to an ephemeral port outside the ports that are usually
	 * "open" in most frameworks.
	 */
	public void run() {
		try {
			Logger log =  Logger.getLogger(DiscoveryService.class.getName());
			
//			final int BUFLEN = 1024;
			final int BUFLEN = 512;
			
//			final int DELAY = 500;   // starts a discovery step every half a second
			final int DELAY = 5000;	// 5 secs
			
			MulticastSocket ms = new MulticastSocket();
			// set a large TTL, possibly means that a larger set of nodes than those on the local network
			// are reached. 
			final int ttl = 150;
			ms.setTimeToLive(ttl);
			
			byte[] discoveryBuffer = PresenceThread.DISCOVERYMESSAGE.getBytes();
			
			InetAddress gia = InetAddress.getByName(PresenceThread.multicastGroup);
			
			//pacchetto con il messaggio di discovery
			DatagramPacket discoveryDp =
				new DatagramPacket(discoveryBuffer, discoveryBuffer.length, gia, PresenceThread.multicastPort);
						
			//pacchetto per la ricezione della risposta
			DatagramPacket dp = new DatagramPacket(new byte[BUFLEN], BUFLEN);
						
			while(true) {				
				//log.debug("New discovery cycle");
				
				//send the discovery packet
				ms.send(discoveryDp);
								
				boolean done = true;
				
				do {
					ms.setSoTimeout(DELAY);
					try {		
						
						ms.receive(dp);
						
						ms.setSoTimeout(0);
						String fullHost = new String(dp.getData(), 0, dp.getLength());
						// patch windows way of giving names to machines with
						// DHCP
						String[] hostNames = fullHost.split("/");
						String host = hostNames[1];
						
						service.addWorker(host);
												
						log.debug("Message received is: >>"+fullHost+"<<");
						
					} catch (SocketTimeoutException e) {
						done = false;
					}
				} while (done);
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		return;
	}

}