/* This is the muskel: a skeleton/RMI based, parallel programming library
Copyright (C) 2006  Marco Danelutto, Dept. Computer Science, Univ. Pisa, Italy

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */

package muskel;

import java.io.Serializable;
import muskel.mdf.GraphId;
import muskel.mdf.InstructionId;
import muskel.mdf.Mdfi;

/**
 * This is used to hash the instructions in the pool
 * 
 * @author marcod
 * 
*/
public class InstructionTag implements Serializable {

	private static final long serialVersionUID = 1L;
	
	InstructionId it = Mdfi.NoInstrId;
	GraphId gt = Mdfi.NoGraphId;
	
	public InstructionTag(InstructionId i, GraphId g) {
		it = i; 
		gt = g;
	}
	
	public InstructionId getInstrId() {
		return it;
	}
	
	public GraphId getGraphId() {
		return gt;
	}

	public String toString() {
		return (it+","+gt);
	}
	
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof InstructionTag))
			return false;
		final InstructionTag other = (InstructionTag) obj;
		if (gt == null) {
			if (other.gt != null)
				return false;
		} else if (!gt.equals(other.gt))
			return false;
		if (it == null) {
			if (other.it != null)
				return false;
		} else if (!it.equals(other.it))
			return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((gt == null) ? 0 : gt.hashCode());
		result = prime * result + ((it == null) ? 0 : it.hashCode());
		return result;
	}
}
