/* This is the muskel: a skeleton/RMI based, parallel programming library
   Copyright (C) 2006  Marco Danelutto, Dept. Computer Science, Univ. Pisa, Italy

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */

/*
 * Created on Mar 24, 2004
 *
 * To change the template for this generated file go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
package muskel;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.InetAddress;
import java.net.MulticastSocket;
import org.apache.log4j.Logger;

/**
 * This is the thread run by the RMI RemoteWorker server to answer the request for presence
 * issued by other StandardEval components.<br>
 * It listens to a Multicast group and answers with the name of the machine where it
 * is running. The asking thread can therefore manage a new worker at that machine.
 * @author Danelutto Marco
 *
 */
public class PresenceThread extends Thread {
	
    /** this is the multicast group of the muskel package discovery service */
    public static final String multicastGroup = "236.7.8.99";
    /** this is the port of the muskel discovery service */
    public static final int multicastPort = 54321;
    /** this is the muskel discovery message
     * (ASCII protocol: send discovery message,
     * receive name of the machine */
    public static final String DISCOVERYMESSAGE = "muskelDiscoveryMessage";
	
    /**
     * the actual thread body: runs forever.
     * Waits for a discovery message. Answers with a message hosting
     * the host name where this server is being run
     */
    public void run() {
    	try {
    		Logger log =  Logger.getLogger(RemoteInterpreter.class.getName());
    		log.debug("Started");

    		MulticastSocket ms  =  new MulticastSocket(multicastPort);
    		InetAddress gia     =  InetAddress.getByName(multicastGroup);
    		ms.joinGroup(gia);

    		final int MAXBUF = 512;
    		byte[] buffer = new byte[MAXBUF];

    		DatagramPacket dp = new DatagramPacket(buffer,MAXBUF);

    		log.debug("Entering the main loop");

    		while (true) {

    			ms.receive(dp);

    			log.debug("Received discovery message from "+dp.getAddress().toString());
    			// discovery protocol requires a "DISCOVERY" string here

    			String message = new String(dp.getData(),0,dp.getLength());

    			log.debug("Message received is: >>"+message+"<<");

    			if(message.compareTo(DISCOVERYMESSAGE) == 0) {
    				// answer with my address 
    				InetAddress myIa = InetAddress.getLocalHost();
    				String myName = myIa.toString(); // myIa.getHostName();
    				byte [] myNameBuf = myName.getBytes();
    				dp.setData(myNameBuf);

    				ms.send(dp); // back to the sender

    				log.debug("Sent answer back to "+dp.getAddress().toString()+" = "+myName);
    			}

    			//log.debug("New receive buffer");
    			dp.setData(new byte[MAXBUF]);//nuovo buffer

    		}

    	} catch (IOException e) {
    		e.printStackTrace();
    	}
    }
}