/* This is the muskel: a skeleton/RMI based, parallel programming library
Copyright (C) 2006  Marco Danelutto, Dept. Computer Science, Univ. Pisa, Italy

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */

package muskel;

/**
 * this is the record used to report statistics from the remote nodes. 
 * It is sent when the control thread associated to the remote node
 * terminates due to termination of the distributed MDF interpreter 
 * (no more fireable instructions, no more tokens on the input stream)
 * 
 */
import java.io.Serializable;

/**
 * @author marcod
 *
 */

public class Stats implements Serializable {

	private static final long serialVersionUID = 4438873568941147196L;
	
	int mdfiNo; // number of instructions executed since last stat reset
	long averageTaskTc; // average number of milliseconds spent in evaluating a task
	long minTimeCompute; // the min time spent computing a task
	long maxTimeCompute; // the max time spent computing a task
	String workerName;

	public void setMdfiNo(int i) { mdfiNo = i; }
	public int getMdfiNo() { return mdfiNo; }

	public void setAverageTaskTc(long t) { averageTaskTc = t; }
	public long getAverageTaskTc() { return averageTaskTc; }

	public void setMinTimeCompute(long t) { minTimeCompute = t; }
	public long getMinTimeCompute() { return minTimeCompute; }

	public void setMaxTimeCompute(long t) { maxTimeCompute = t; }
	public long getMaxTimeCompute() { return maxTimeCompute; }

	public void setWorkerName(String s) { workerName = s; }
	public String getWorkerName() { return workerName; }

	public String toString() {
		String r = "stats for "+workerName + ": ";
		r = r + " mdfiNo=" + (new Integer(mdfiNo)).toString(); 
		r = r + " avgTc=" + (new Long(averageTaskTc)).toString();
		r = r + " minTc=" + (new Long(minTimeCompute)).toString();
		r = r + " maxTc=" + (new Long(maxTimeCompute)).toString();
		return r;
	}
}
