package muskel.mdf;

import java.io.Serializable;

/**
 * Un Task utilizzabile come valore di un Token
 * (sia di input che di output)
 * L'Object contenuto DEVE essere immutabile,
 * per permettere la clone() (di tipo shallow copy).
 * 
 * @author albanese
 * */
public class Task implements Cloneable, Serializable {

	private static final long serialVersionUID = -3987064469265405519L;
	
	private Object value;

	public Task(Object value) {
		this.value = value;
	}
	
	public Object getValue() {
		return value;
	}
	
	/**
	 * Clona l'oggetto; esso deve essere immutabile
	 * in quanto viene eseguita una shallow copy
	 * */
	public Object clone() {
		
		return new Task(this.value);
	}
	
	public String toString() {
		
		return value.toString();
	}
}
