package muskel.mdf;

import java.io.Serializable;
import muskel.TokenNotPresentException;

/**
 * Un Token contentente un task e un bit di presenza.
 * 
 * @author albanese
 * */
public class Token implements Serializable, Cloneable {

	private static final long serialVersionUID = 1L;
	
	protected Task task = null; 
	protected boolean presenceBit = false;
	
	/**
	 * Creates a token with a given value
	 * @param task The value to be stored in the token
	 */
	public Token(Task task) {
		this.task = task; 
		presenceBit = true;
	}
	
	/** 
	 * This is to create an empty token, just a placeholder
	 */
	public Token() {
		task = null; 
		presenceBit = false;
	}
	
	/** 
	 * This is to store a task in an empty token
	 * or in an already setup token
	 * @param task The new task for the token
	 */
	public void updateToken(Task task) {
		this.task = task;
		presenceBit = true;
	}
	
	/**
	 * @return Il task contenuto nel Token
	 * @throws TokenNotPresentException Se il task
	 * 			non e' presente (il bit di presenza e' false).
	 * 
	 * */
	public Task getTask() throws TokenNotPresentException {
		if(presenceBit) 
			return task;
		else 
			throw new TokenNotPresentException();
	}
	
	public String toString() {
		if(task == null) 
			return("<null,"+presenceBit+">");
		else
			return("<"+task.toString()+","+presenceBit+">");
	}
	
	/**
	 * deep copy
	 * */
	public Object clone() {
		
		Task clonedTask =
			(task == null) ? null : (Task) task.clone();
				
		Token clone = new Token();
		clone.task = clonedTask;
		clone.presenceBit = this.presenceBit;
		
		return clone;
	}
}
