/* This is the muskel: a skeleton/RMI based, parallel programming library
 Copyright (C) 2006  Marco Danelutto, Dept. Computer Science, Univ. Pisa, Italy

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */

package muskel.stream;

import org.apache.log4j.Logger;

/**
 * 
 * Sample input manager. Delivers a number of Integer items onto the input
 * stream, when the next() method is invoked from within the manager
 * 
 * @author marcod
 * 
 */
public class IntegerStreamInputManager implements NonBlockingInputManager {

	private int howMuch;

	private Logger log = Logger.getLogger(IntegerStreamInputManager.class
			.getName());

	/**
	 * creates a new IntegerStreamInputManager
	 * 
	 * @param ithe number of items to be delivered on the input stream upon
	 *            Manager request.
	 * 
	 */
	public IntegerStreamInputManager(int i) {
		howMuch = i;
	}

	/**
	 * delivers next item
	 * 
	 * @return the new stream item (an Integer)
	 */
	public Object next() {
		if (howMuch > 0) {
			log.debug("delivered input task : " + howMuch);
			return new Integer(howMuch--);
		} else {
			return null;
		}
	}

	/**
	 * @return true if there are other items to be delivered onto the input
	 *         stream, false otherwise
	 */
	public boolean hasNext() {
		if (howMuch == 0) {
			return false;
		} else {
			return true;

		}
	}
}