/* This is the muskel: a skeleton/RMI based, parallel programming library
 Copyright (C) 2006  Marco Danelutto, Dept. Computer Science, Univ. Pisa, Italy

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */

package muskel.stream;

import java.util.Random;
import java.util.Vector;
import org.apache.log4j.Logger;

/**
 * 
 * Sample input manager. Delivers Integer Vector
 * (having the specified dimension) items onto the input
 * stream, when the next() method is invoked from within the manager
 * 
 * @author albanese
 * 
 */
public class IntegerVectorStreamInputManager implements NonBlockingInputManager {

	private int howMuch;
	private int dimension;
	
	private Random randomGenerator;

	//i numeri casuali generati sono in [0, maxInt-1]
	private static final int MAX_INT = 10001;
	
	private Logger log = Logger.getLogger(IntegerVectorStreamInputManager.class
			.getName());

	/**
	 * creates a new IntegerVectorStreamInputManager
	 * 
	 * @param howMuch
	 *            the number of items to be delivered on the input stream upon
	 *            Manager request.
	 * @param dimension The dimension of vector to generate
	 */
	public IntegerVectorStreamInputManager(int dimension, int howMuch) {
		this.dimension = dimension;
		this.howMuch = howMuch;
		
		randomGenerator = new Random();
	}

	/**
	 * delivers next item
	 * 
	 * @return the new stream item (an Integer)
	 */
	public Object next() {
		if (howMuch > 0) {
			log.debug("delivered input task : " + howMuch);
						
			howMuch--;
			return generateVector();
		} else {
			return null;
		}
	}

	private Vector<Integer> generateVector() {
		
		Vector<Integer> v = new Vector<Integer>(dimension);	
		
		for(int i = 0; i < dimension;i++)
			v.add(randomGenerator.nextInt(MAX_INT));
		
		return v;
	}

	/**
	 * @return true if there are other items
	 * 			to be delivered onto the input stream,
	 * 			false otherwise
	 */
	public boolean hasNext() {
		if (howMuch == 0) {
			return false;
		} else {
			return true;

		}
	}

}
