/* This is the muskel: a skeleton/RMI based, parallel programming library
Copyright (C) 2006  Marco Danelutto, Dept. Computer Science, Univ. Pisa, Italy

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */

/*
 * Created on Mar 22, 2004
 *
 * To change the template for this generated file go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
package skeleton;

import java.io.Serializable;
import muskel.mdf.Task;

/**
  * This is the interface that has to be implemented by
  * the sequential portions of codeused in the program computation.
  *	Each sequential skeleton is an object that implements this interface. 
  * We explicitly chose not to use 1.5 features, at the moment.
  * <br>
  * The typical usage is the following: 
  * <ul>
  * <li> suppose you have to compute in parallel f(x1) ... f(xn)
  *		 out of x1 ... xn<br>
  * <li> suppose f takes as input an InputTask task
  * 	 and produces and OutputResult result
  * <li> then you'll use a code such as:
  * <pre>
  * public class myF extends Compute {
  *    public Task[] compute(Task[] task) {
  *      OutputResult res = f((InputTask) task);
  *      Task[] toReturn = {new Task(res)};
  *      return toReturn;
  *    }
  * }
  * </pre>
  * every time you need to use the f function to compute
  * a pipeline stage, a farm worker, etc.
  * </ul>
  * 
  * @author Danelutto Marco
 */
public abstract class Compute implements Cloneable, Serializable  {
	/**
	 * the method computes a Task[] result out of a Task[] input data set
	 * @return the result computed 
	 */
	public abstract Task[] compute(Task[] task);
	
	/**
	 *  Just to be able to clone in case of usage of ObjectStreams
	 *  Shallow copy (the Compute Object are immutable)
	 * @return this object
	 */
	public Object clone() {
		
		return this;
	}
}
