/* This is the muskel: a skeleton/RMI based, parallel programming library
Copyright (C) 2006  Marco Danelutto, Dept. Computer Science, Univ. Pisa, Italy

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */

/*
 * Created on Mar 22, 2004
 *
 * To change the template for this generated file go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
package skeleton;

import muskel.mdf.Task;


/**
 * This is the pipeline skeleton class.
 * The muskel pipeline is a two stage pipeline.
 * Multiple stage pipelines can be defined nesting
 * two stage pipelines. That is, provided that
 * <code>StageX</code> are all skeletons
 * (i.e. they implement the Compute interface)
 * the following code represents a three stage pipeline:
 * <pre>
 * Pipeline p1 = new Pipeline(Stage1, Stage2);
 * Pipeline main = new Pipeline(p1, Stage3);
 * </pre>
 * 
 * This is for the sake of simplicity. There is no
 * problem in implementing an n-stage pipeline,
 * using a Vector of stages instead of a fixed set
 * of variables hosting the stages.
 * <p>
 * 
 * @author Danelutto Marco
 *
 */
public class Pipeline extends Compute {
	
	private static final long serialVersionUID = 1L;
	
	Compute stage1 = null;
	Compute stage2 = null;
	
	/**
	 * constructs a two stage pipeline
	 * @param stage1 is the skeleton representing
	 * 			the computation to be performed at the first stage
	 * @param stage2 is the skeleton representing
	 * 			the computation to be performed at the second stage
	 */
	public Pipeline(Compute stage1, Compute stage2) {
		this.stage1 = stage1;
		this.stage2 = stage2;
	}
	
	/**
	 * This returns the first stage, used in the MDF conpiler
	 * @return The first stage
	 */
	public Compute getFirstStage() {
		return stage1;
	}
	/**
	 * this returns the second stage, used in the MDF compiler
	 * @return The second stage
	 */
	public Compute getSecondStage() {
		return stage2;
	}
	/**
	 * computes the pipeline sequentially.
	 * @param task is the data input to be processed by the pipeline
	 * @return the result computed by the pipeline as a Task[]
	 */
	public Task[] compute(Task[] task) {
		Task[] stage1Result = stage1.compute(task);
		
		return stage2.compute(stage1Result);
	}
}
